import csv
import json
from one_py_sdk.operations.spreadsheet import SpreadsheetApi
from one_py_sdk.common.library import LibraryApi
from one_py_sdk.enterprise.twin import DigitalTwinApi
from one_py_sdk.shared.helpers.datetimehelper import *
from datetime import datetime, timezone

class Exporter:
    def __init__(self, env, auth):
        self.Environment = env
        self.Authentication = auth
        self.Spreadsheet = SpreadsheetApi(env, auth)
        self.Library = LibraryApi(env, auth)
        self.DigitalTwin = DigitalTwinApi(env, auth)
                                                      
    def ExportWorksheet(self, filename, plantId, startDate, endDate, updatedAfter=None, wsType=None): 
        with open(filename, mode='w', newline='', encoding="utf-8") as file:        
            fieldnames = ['Worksheet Type', 'Time', 'ColumnName','ColumnId','RowNumber', 'Value', 'StringValue', 'DateEntered']                
            worksheetWriter =csv.DictWriter(file,fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:                    
                wsTypes = range(1,5)
                for wsType in wsTypes:
                    try: 
                         self.__mapAndWriteRowsAndColumns(worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter)                    
                    except:                        
                        continue                    
            else:
                self.__mapAndWriteRowsAndColumns(worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter)                                           
    
    def __mapAndWriteRowsAndColumns(self, worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter):
        wsVal =self.ConvertWSTypeToStringValue(wsType)                       
        try:
            ws=self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[0]                  
        except:
            return print("No worksheet definition found")                  
        if not ws.columns:
            return print("No columns found")
        rows =self.Spreadsheet.GetRowsForTimeRange(plantId, wsType, startDate, endDate)
        try:
            rowNumbers = rows.keys()
            rowValues = rows.values()  
        except(AttributeError):
            return print("No rows found")                      
        rowDict ={}
        for num in rowNumbers:
            rowDict[num]= str(GetDateFromRowNumber(num, ws.enumWorksheet))
        
        numberMapping = {}
        for column in ws.columns:
            numberMapping[column.columnNumber] = [column.name, column.columnId]  
        for vals in rows.values():            
            for cell in vals.cells:                                                        
                if updatedAfter!=None:
                    try: 
                        dateEntered =cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value
                        dateEntered =datetime.strptime(dateEntered[:-2], '%Y-%m-%dT%H:%M:%S.%f')
                        dateEntered =dateEntered.replace(tzinfo=timezone.utc)
                        if dateEntered> updatedAfter:
                            worksheetWriter.writerow({'Worksheet Type': wsVal, 'Time': rowDict[vals.rowNumber],'ColumnName':numberMapping[cell.columnNumber][0],
                                                'ColumnId':numberMapping[cell.columnNumber][1],'Value': (cell.cellDatas[0].value.value),
                                                'RowNumber':vals.rowNumber, 'StringValue':cell.cellDatas[0].stringValue.value, 
                                                'DateEntered':cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value})
                    except(IndexError):                    
                        pass
                else:
                    try:
                            worksheetWriter.writerow({'Worksheet Type': wsVal, 'Time': rowDict[vals.rowNumber],'ColumnName':numberMapping[cell.columnNumber][0],
                                                'ColumnId':numberMapping[cell.columnNumber][1],'Value': (cell.cellDatas[0].value.value),
                                                'RowNumber':vals.rowNumber, 'StringValue':cell.cellDatas[0].stringValue.value, 
                                                'DateEntered':cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value})
                    except(IndexError):                    
                        pass
                                                                    
    def ExportWorksheetByType(self, filename, plantId, startDate, endDate, wsType=4, updatedAfter=None):               
            wsVal =self.ConvertWSTypeToStringValue(wsType)  
            with open(filename, mode='w', newline='', encoding="utf-8") as file:        
                fieldnames = ['Worksheet Type', 'Time', 'ColumnName','ColumnId','RowNumber', 'Value', 'StringValue', 'DateEntered']                
                worksheetWriter =csv.DictWriter(file,fieldnames=fieldnames)
                worksheetWriter.writeheader()                                       
                self.__mapAndWriteRowsAndColumns(worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter)

    def ExportColumnDetails(self, filename, plantId, wsType =None):                    
        unitDict, paramDict,typeDict, subtypeDict = self.__mapUnitsAndParams()
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = ['Worksheet Type','ColumnNumber', 'Name', 'ParameterId','LocationId','LocationName','LocationType', 'LocationSubtype', 'Path', 'ParameterTranslation','ColumnId', 'UnitId', 'UnitTranslation','LastPopulatedDate', 'LimitName',"LowValue", "LowOperation", "HighValue", "HighOperation", "LimitStartTime", "LimitEndTime" ]        
            worksheetWriter =csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:
                wsTypes = range(1,5)
                for wsType in wsTypes:
                    try:
                        self.__mapAndWriteColumns(plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict)
                    except: 
                        raise
                        continue
            else: 
                self.__mapAndWriteColumns(plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict)
     
    def ExportLimitColumns(self, filename, plantId, wsType =None):                    
        unitDict, paramDict,typeDict, subtypeDict = self.__mapUnitsAndParams()
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = ['Worksheet Type','ColumnNumber', 'Name', 'ParameterId','LocationId','LocationName','LocationType', 'LocationSubtype', 'Path', 'ParameterTranslation','ColumnId', 'UnitId', 'UnitTranslation','LastPopulatedDate', 'LimitName',"LowValue", "LowOperation", "HighValue", "HighOperation", "LimitStartTime", "LimitEndTime" ]        
            worksheetWriter =csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:
                wsTypes = range(1,5)
                for wsType in wsTypes:
                    try:
                        self.__mapAndWriteLimitColumns(plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict)
                    except:                         
                        continue
            else: 
                self.__mapAndWriteLimitColumns(plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict)  
    
    def __mapAndWriteLimitColumns(self, plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict):
        wsVal = self.ConvertWSTypeToStringValue(wsType)
        try:
            ws=self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[0]                              
        except:
            return print("No worksheet definition found")
        if not ws.columns:
            return print("No columns found")             
        
        columnDict = {}
        for column in ws.columns:                    
            columnDict[column.columnNumber] = [column.name,  column.columnId, column.parameterId,  column.displayUnitId, column.lastRowNumberWithData, column.locationId, column.limits]       
        limitDict ={}
        for column in columnDict.values():            
            try:
                limit = column[6][0]
                if limit.enumLimit ==5 or limit.enumLimit == 4 or limit.enumLimit == 1:
                    limitDict[column[1]]= [limit.name, limit.lowValue.value, limit.lowOperation, limit.highValue.value, limit.highOperation,  limit.timeWindow.startTime.jsonDateTime.value, limit.timeWindow.endTime.jsonDateTime.value ]
                                 
            except:
                pass
           
        twinDict =self.PathFinder(plantId, columnDict)        
        for key in columnDict.keys():                             
            try:
                locationSubtype = subtypeDict[twinDict[columnDict[key][5]][5]][1]
                locationType =typeDict[twinDict[columnDict[key][5]][4]][1]      
                worksheetWriter.writerow({ 'ColumnNumber': key, 
                                        'Name':columnDict[key][0], 
                                        'ColumnId': columnDict[key][1], 
                                        'Worksheet Type':wsVal,
                                        'ParameterId':columnDict[key][2],
                                        'UnitId': columnDict[key][3],
                                        'LocationName': twinDict[columnDict[key][1]][3],
                                        'LastPopulatedDate': GetDateFromRowNumber(columnDict[key][4], wsType),
                                        'UnitTranslation':unitDict[columnDict[key][3]][2],
                                        'LocationId':columnDict[key][5],
                                        'LocationType':locationType,
                                        'ParameterTranslation':paramDict[columnDict[key][2]][1], 
                                        "Path": twinDict[columnDict[key][1]][2],
                                        'LimitName' :  limitDict[columnDict[key][1]][0], 
                                        "LowValue": limitDict[columnDict[key][1]][1], 
                                        "LowOperation": self.LimitOperationToStringValue(limitDict[columnDict[key][1]][2]),
                                        "HighValue": limitDict[columnDict[key][1]][3], 
                                        "HighOperation":self.LimitOperationToStringValue(limitDict[columnDict[key][1]][4]),
                                        "LimitStartTime":limitDict[columnDict[key][1]][5],
                                        "LimitEndTime":limitDict[columnDict[key][1]][6],
                                        "LocationSubtype": locationSubtype
                                                                                                                                
                                        })
            except (KeyError):
                pass
            
    def __mapAndWriteColumns(self, plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict):
        wsVal = self.ConvertWSTypeToStringValue(wsType)
        try:
            ws=self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[0]                              
        except:
            return print("No worksheet definition found")
        if not ws.columns:
            return print("No columns found")             
        
        columnDict = {}
        for column in ws.columns:                    
            columnDict[column.columnNumber] = [column.name,  column.columnId, column.parameterId,  column.displayUnitId, column.lastRowNumberWithData, column.locationId, column.limits]       
        limitDict ={}
        for column in columnDict.values():            
            try:
                limit = column[6][0]
                if limit.enumLimit ==5 or limit.enumLimit == 4 or limit.enumLimit == 1:
                    limitDict[column[1]]= [limit.name, limit.lowValue.value, limit.lowOperation, limit.highValue.value, limit.highOperation,  limit.timeWindow.startTime.jsonDateTime.value, limit.timeWindow.endTime.jsonDateTime.value ]
                                 
            except:
                pass
           
        twinDict =self.PathFinder(plantId, columnDict)        
        for key in columnDict.keys():
            locationType =typeDict[twinDict[columnDict[key][5]][4]][1]            
            
            try:
                locationSubtype = subtypeDict[twinDict[columnDict[key][5]][5]][1]
                worksheetWriter.writerow({ 'ColumnNumber': key, 
                                        'Name':columnDict[key][0], 
                                        'ColumnId': columnDict[key][1], 
                                        'Worksheet Type':wsVal,
                                        'ParameterId':columnDict[key][2],
                                        'UnitId': columnDict[key][3],
                                        'LocationName': twinDict[columnDict[key][1]][3],
                                        'LastPopulatedDate': GetDateFromRowNumber(columnDict[key][4], wsType),
                                        'UnitTranslation':unitDict[columnDict[key][3]][2],
                                        'LocationId':columnDict[key][5],
                                        'LocationType':locationType,
                                        'ParameterTranslation':paramDict[columnDict[key][2]][1], 
                                        "Path": twinDict[columnDict[key][1]][2],
                                        'LimitName' :  limitDict[columnDict[key][1]][0], 
                                        "LowValue": limitDict[columnDict[key][1]][1], 
                                        "LowOperation": self.LimitOperationToStringValue(limitDict[columnDict[key][1]][2]),
                                        "HighValue": limitDict[columnDict[key][1]][3], 
                                        "HighOperation":self.LimitOperationToStringValue(limitDict[columnDict[key][1]][4]),
                                        "LimitStartTime":limitDict[columnDict[key][1]][5],
                                        "LimitEndTime":limitDict[columnDict[key][1]][6],
                                        "LocationSubtype": locationSubtype
                                                                                                                                
                                        })
            except (KeyError):
                locationSubtype = subtypeDict[twinDict[columnDict[key][5]][5]][1]
                worksheetWriter.writerow({ 'ColumnNumber': key, 
                                            'Name':columnDict[key][0], 
                                            'ColumnId': columnDict[key][1], 
                                            'Worksheet Type':wsVal,
                                            'ParameterId':columnDict[key][2],
                                            'UnitId': columnDict[key][3],
                                            'LocationName': twinDict[columnDict[key][1]][3],                                            
                                            'LastPopulatedDate': GetDateFromRowNumber(columnDict[key][4], wsType),
                                            'UnitTranslation':unitDict[columnDict[key][3]][2],
                                            'LocationId':columnDict[key][5],
                                            'LocationType':locationType,
                                            'ParameterTranslation':paramDict[columnDict[key][2]][1], 
                                            "Path": twinDict[columnDict[key][1]][2],
                                            "LocationSubtype": locationSubtype
                                            })
                
    
    def __mapUnitsAndParams(self):
        units = self.Library.GetUnits()  
        parameters = self.Library.GetParameters()       
        i18N = self.Library.Geti18nKeys("AQI_FOUNDATION_LIBRARY")[0].get("AQI_FOUNDATION_LIBRARY")
        i18NUnits= i18N.get("UnitType").get("LONG")
        i18NParams= i18N.get("Parameter").get("LONG")  
        i18NSubtypes= i18N.get("DigitalTwinSubType")   
        paramDict = {}
        for param in parameters:
            try:
                paramDict[param.IntId]= [param.i18nKey, i18NParams[param.i18nKey]]
            except: 
                paramDict[param.IntId]= [param.i18nKey, None]                     
        unitDict = {}
        for unit in units:
            try:
                unitDict[unit.IntId]= [unit.i18nKey, unit.unitName, i18NUnits[unit.i18nKey]]
            except:
                unitDict[unit.IntId]= [unit.i18nKey, unit.unitName, None]
        twinTypes = self.DigitalTwin.GetDigitalTwinTypes()
        typeDict ={}
        for twinType in twinTypes:
            typeDict[twinType.id]=[twinType.i18NKeyName, twinType.description.value]
        twinSubtypes = self.DigitalTwin.GetDigitalTwinSubtypes()
        subTypeDict ={}
        for subtype in twinSubtypes:
            try:
                subTypeDict[subtype.id]=[subtype.i18NKeyName, i18NSubtypes[subtype.i18NKeyName] ]
            except(KeyError):
                subTypeDict[subtype.id]=[subtype.i18NKeyName, subtype.i18NKeyName]
        
        return unitDict, paramDict, typeDict, subTypeDict
    
    def ExportColumnDetailsByType(self, filename, plantId, wsType=4):         
        unitDict, paramDict, type = self.__mapUnitsAndParams()
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = ['Worksheet Type','ColumnNumber', 'Name', 'ParameterId','LocationId','LocationName', 'Path', 'ParameterTranslation','ColumnId', 'UnitId', 'UnitTranslation','LastPopulatedDate', 'LimitName',"LowValue", "LowOperation", "HighValue", "HighOperation", "LimitStartTime", "LimitEndTime" ]        
            worksheetWriter =csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            self.__mapAndWriteColumns(plantId, wsType, unitDict, paramDict, worksheetWriter)
                             
    def PathFinder(self, plantId, columnDict):        
        twins = self.DigitalTwin.GetDescendantsByType(plantId, "ae018857-5f27-4fe4-8117-d0cbaecb9c1e", False)
        twins.MergeFrom(self.DigitalTwin.GetDescendantsByRefByCategory(plantId, 2,False))
        twins= twins.items      
        twinDict ={}
        for twin in twins:
            twinDict[twin.twinReferenceId.value]=[twin.parentTwinReferenceId.value, twin.name.value, None, None, twin.twinTypeId, twin.twinSubTypeId.value]                
        for key in columnDict.keys():
            twinId=columnDict[key][1]
            path =[]
            pathString=""
            while (twinId!=plantId):             
                path.append(twinDict[twinId][1])
                twinId =twinDict[twinId][0]                                
            path.append(twinDict[twinId][1])
            twinDict[columnDict[key][1]][3]=path[1]
            while (path):
                pathString= f'{pathString}/{path.pop()}'
            twinDict[columnDict[key][1]][2]=pathString
        return twinDict
    
    def ConvertWSTypeToStringValue(self, wsType):
        if wsType ==1:
            return "Fifteen Minute"
        elif wsType ==2:
            return "Hourly"   
        elif wsType ==3:
            return "Four Hour"                                  
        elif wsType ==4:
            return "Daily"            
        else: 
             return print("Enter valid worksheet type value (1= Fifteen minute, 2=Hourly, 3=FourHour, 4=Daily)") 
           
    def LimitOperationToStringValue(self, limitOperation):
        if limitOperation ==1:
            return ">"
        elif limitOperation ==2:
            return ">="   
        elif limitOperation ==3:
            return "<"                                  
        elif limitOperation ==4:
            return "=<"       
        elif limitOperation ==0:
            return "Unknown"            
        else: 
             return print(f"{limitOperation}Enter valid limit Operation value (1 is >, 2  is ≥  , 3 is < , 4 is ≤)")      
   
          
            
        


import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
import uuid 


class DigitalTwinApi:
    def __init__(self, env, auth:AuthenticationApi):
        self.AppUrl = "/enterprise/twin/v1/"
        self.Environment = env
        self.Authentication = auth       
        
    def Get(self, twinRefId):
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId        
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))        
        return response.content.DigitalTwins.items
    
    def GetTwinMeasurementsByRefId(self, twinRefId):
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId        
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))              
        return json.loads(response.content.DigitalTwins.items[0].twinData.value).get('measurement')      
    
    def TwinMeasurements(self, twinRefId, twinMeasurements):
        try:         
             usefulDictionary ={"Telemetry Id": twinRefId,
                            "Value": twinMeasurements['value'], 
                            "String Value": twinMeasurements['stringValue'],
                            "Timestamp": twinMeasurements['timestamp'].get('jsonDateTime'),
                            "TelemetryPath": self.findTelemetryPath(twinRefId)}
             return usefulDictionary
        except (TypeError):
             return ("No twin data found for "+twinRefId)       
        
    def GetDigitalTwinTypes(self):
        requestId =uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwinType?requestId="+str(requestId)
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))   
        return response.content.digitalTwinTypes.items
    
    def GetDigitalTwinType(self, twinTypeId):
        requestId =uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}DigitalTwinType/{twinTypeId}?requestId={str(requestId)}'
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))   
        return response.content.digitalTwinTypes.items


    def GetDigitalTwinSubtypes(self):
        requestId =uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwinSubType?requestId="+str(requestId)
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))   
        return response.content.digitalTwinSubtypes.items
    
    def GetDescendantsByType(self, twinRefId:str, twinTypeId:str, items=True):
        requestId =uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId+"/Type/"+twinTypeId+"/Descendants?requestId="+str(requestId)        
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))        
        if items:
            return response.content.DigitalTwins.items
        else:
            return response.content.DigitalTwins    
    def GetDescendantsBySubType(self, twinRefId:str, twinSubtypeId:str):
        requestId =uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId+"/Subtype/"+twinSubtypeId+"/Descendants?requestId="+str(requestId)        
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))        
        return response.content.DigitalTwins.items
    
    def GetDescendants(self, twinRefId:str):
        requestId =uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId+"/Descendants?requestId="+str(requestId)        
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))        
        return response.content.DigitalTwins.items
    
    def GetDescendantsByRefByCategory(self, twinRefId:str, categoryId:int, items=True):
        requestId =uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}DigitalTwin/Ref/{twinRefId}/Category/{categoryId}/Descendants?requestId={str(requestId)}'        
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))        
        if items:
            return response.content.DigitalTwins.items
        else:
            return response.content.DigitalTwins
    
    def findTelemetryPath(self, twinRefId):
        statusCode=""
        twinPath=[]        
        while (True):
            url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+str(twinRefId)                         
            headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}
            response = DeserializeResponse(requests.get(url, headers=headers))
            if (response.statusCode!=200):
                break                                                                          
            twinRefId =response.content.DigitalTwins.items[0].parentTwinReferenceId.value            
            twinPath.append(response.content.DigitalTwins.items[0].name.value)            
        twinPathString=""
        while (len(twinPath)>0):
            twinPathString = twinPathString+str(twinPath.pop())+"/"
        return twinPathString
  
                
        

    
    


import requests
import json
from logging import Logger
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class LibraryApi:
    def __init__(self, env, auth):
        self.AppUrl = "/common/library/v1/"
        self.Environment = env
        self.Authentication = auth
    
    def GetUnits(self):
        url = self.Environment+self.AppUrl+"unit"     
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))   
        return response.content.units.items
    
    def getParameter(self, parameterId):
        pass
    
    def GetParameters(self):
        url = self.Environment+self.AppUrl+"parameter"     
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        r =requests.get(url, headers=headers)        
        response = DeserializeResponse(r)
        
        return response.content.parameters.items
    
    def GetQuantityTypes(self):
        url = self.Environment+self.AppUrl+"quantityType"     
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))   
        return response.content.quantityTypes.items
    
    def Geti18nKeys(self,  modules:str, language:str="en",):
        url = self.Environment+self.AppUrl+"i18n"  
        if (language and modules):
            url = url+"?modulecsv="+modules+"&lang="+language       
        elif (modules):
            url = url+"?modulecsv="+modules   
        else:
            return print("Modules is a required parameter")     
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept": "application/json"}
        response = requests.get(url, headers=headers)
        jResponse= json.loads(response.content)
        return jResponse.get("FM")


Environment = {"feature":"https://api-feature-us.aquaticinformatics.net", "integration":"https://api-integration-us.aquaticinformatics.net", "stage":"https://api-stage-us.aquaticinformatics.net", "production":"https://api-us.aquaticinformatics.net/"}



from imp import cache_from_source
import requests
from datetime import time
import json
import requests_cache
from one_py_sdk.enterprise.core import CoreApi
from one_py_sdk.historian.data import HistorianApi
from one_py_sdk.operations.spreadsheet import SpreadsheetApi
from one_py_sdk.common.library import LibraryApi
from one_py_sdk.enterprise.twin  import DigitalTwinApi
from one_py_sdk.shared.constants import *
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.csvhelper import Exporter



class ClientSdk:
	def __init__(self, env="https://api-us.aquaticinformatics.net/", cacheTimeout=None):
		self.Environment = env
		self.Initialize(cacheTimeout)
	
	def Initialize(self, cacheTimeout):
		self.Authentication= AuthenticationApi(self.Environment)
		self.DigitalTwin = DigitalTwinApi(self.Environment, self.Authentication)
		self.Spreadsheet = SpreadsheetApi(self.Environment, self.Authentication)
		self.Library = LibraryApi(self.Environment, self.Authentication)
		self.Core = CoreApi(self.Environment, self.Authentication)
		self.Historian = HistorianApi(self.Environment, self.Authentication)
		self.Exporter = Exporter(self.Environment, self.Authentication)
		if cacheTimeout!=None:
			requests_cache.install_cache("client-cache", backend="memory", expire_after=cacheTimeout)

	
	def LoadCurrentUser(self):
		if not self.Authentication.IsAuthenticated:
			print("Not authenticated. Authenticate and try again")
		if(self.Authentication.User.id != None):
			self.Authentication.GetUserInfo()
			self.Authentication.User.CopyFrom(self.Core.GetUser(self.Authentication.User.id))


  

	
 


from datetime import datetime, timezone, timedelta
from one_interfaces import jsonTicksDateTime_pb2 as jsonTicksTime


BaseTime = datetime(1900, 1, 1, 0, 0, 0, 0, timezone.utc)

def GetRowNumber(date:datetime, wsType):
    diffTime = date- BaseTime
    windowSize = TimeSpanOfWorksheetType(wsType)
    diffTimeMinutes = diffTime.total_seconds()/60
    windowSizeMinutes =windowSize.total_seconds()/60
    return int(diffTimeMinutes/ windowSizeMinutes) +1 

def GetDateFromRowNumber(rowNumber, wsType):
    row =rowNumber-1
    windowSize = TimeSpanOfWorksheetType(wsType)
    windowSizeMinutes =windowSize.total_seconds()/60
    mins = windowSizeMinutes*row
    return BaseTime+timedelta(minutes = mins)

def ToJsonTicksDateTime(date: datetime):
    jsTime = jsonTicksTime.JsonTicksDateTime()
    jsTime.jsonDateTime.value = str(date)
    return jsTime
    
def TimeSpanOfWorksheetType(wsType):
    if wsType == 1:
        return timedelta(minutes= 15)
    elif wsType == 2:
        return timedelta(hours= 1)
    elif wsType == 3:
        return timedelta(hours= 4)
    elif wsType == 4:
        return timedelta(days= 1)
    else:
        return "Invalid worksheet type"


from one_interfaces import apierror_pb2 as apiError
from one_interfaces import apiresponse_pb2 as apiResponse
import google
import logging

def DeserializeResponse(response):                
      try:
            pbResponse = apiResponse.ApiResponse()                                    
            pbResponse.ParseFromString(response.content)
            return pbResponse
      except  Exception as Argument:
            logging.exception("Error occured in deserialization process")
            pass
     
     
            
    
      
    


from datetime import datetime
import uuid
import requests
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_py_sdk.shared.helpers.datetimehelper import *
from one_interfaces import row_pb2 as row
class SpreadsheetApi:
    def __init__(self, env: str, auth: AuthenticationApi):
        self.Environment = env
        self.Auth = auth
        self.AppUrl ="/operations/spreadsheet/v1/"
    
    def GetWorksheetColumnIds(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + "/worksheet/"+str(wsType)+"/definition"
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers))   
        if response.errors:            
            return response     
        columnIds = [col.columnId for col in response.content.worksheetDefinitions.items[0].columns if  col.isActive==True]        
        return columnIds
    
    def GetWorksheetColumnNumbers(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + "/worksheet/"+str(wsType)+"/definition"
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers)) 
        if response.errors:            
            return response       
        columnNumbers = [col.columnNumber for col in response.content.worksheetDefinitions.items[0].columns if  col.isActive==True]                
        return columnNumbers
    
    def GetWorksheetDefinition(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + "/worksheet/"+str(wsType)+"/definition"
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers))    
        if response.errors:            
            return response           
        return response.content.worksheetDefinitions.items
    
    def GetSpreadsheetDefinition(self, plantId):
        url = f'{self.Environment}{self.AppUrl}{plantId}/definition'
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers))               
        return response.content.spreadsheetDefinitions.items
    
    def GetColumnByDay(self, plantId, wsType, columnId, date:datetime):
        url = self.Environment + self.AppUrl + plantId + f'/worksheet/{str(wsType)}/column/{columnId}/byday/{date.year}/{date.month}/{date.day}'        
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}       
        response =DeserializeResponse(requests.get(url, headers=headers))          
        if response.errors:            
            return response
        return response.content.measurements.items   
    
    def GetColumnByMonth(self, plantId:str, wsType:int, columnId:int, date:datetime):
        url = self.Environment + self.AppUrl + plantId + f'/worksheet/{str(wsType)}/column/{columnId}/bymonth/{date.year}/{date.month}'        
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers))  
        if response.errors:            
            return response
        return response.content.measurements.items  
    
    def GetColumnByYear(self, plantId, wsType, columnId, date:datetime):
        url = self.Environment + self.AppUrl + plantId + f'/worksheet/{str(wsType)}/column/{columnId}/byyear/{date.year}'        
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers))  
        if response.errors:            
            return response
        return response.content.measurements.items  
            
    def GetRows(self, plantId, wsType, startRow=None, endRow=None, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        requestId = uuid.uuid4()
        url=f'{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows?requestId={requestId}'
        if startRow:
            url=url+f'&startRow={startRow}'
        if endRow:
            url=url+f'&endRow={endRow}'
        if columns:
            url=url+f'&columns={columns}'
        if viewId:
            url=url+f'&viewId={viewId}'        
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers)) 
        if response.errors:            
            return response
        return response.content.rows.items
    
             
    def __getRows(self, plantId, wsType, startRow=None, endRow=None, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        requestId = uuid.uuid4()
        url=f'{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows?requestId={requestId}'
        if startRow:
            url=url+f'&startRow={startRow}'
        if endRow:
            url=url+f'&endRow={endRow}'
        if columns:
            url=url+f'&columns={columns}'
        if viewId:
            url=url+f'&viewId={viewId}'        
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers)) 
        if response.errors:            
            return response
        return response.content.rows
    
    def GetRowsByDay(self, plantId, wsType, date:datetime, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        url = self.Environment + self.AppUrl +f'{plantId}/worksheet/{str(wsType)}/rows/byday/{date.year}/{date.month}/{date.day}'  
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers)) 
        if response.errors:            
            return response
        return response.content.rows.items
    
    def GetRowsByMonth(self, plantId, wsType, date:datetime, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        url = self.Environment + self.AppUrl +f'{plantId}/worksheet/{str(wsType)}/rows/bymonth/{date.year}/{date.month}'  
        headers = {'Authorization': self.Auth.Token.access_token, "Accept":"application/x-protobuf"}
        response =DeserializeResponse(requests.get(url, headers=headers)) 
        if response.errors:            
            return response
        return response.content.rows.items
    
    def GetRowsForTimeRange(self, plantId, wsType, startDate:datetime, endDate:datetime):
        startRow =GetRowNumber(startDate, wsType)
        endRow =GetRowNumber(endDate, wsType)       
        rows = row.Rows()
        while endRow -startRow>5000:            
            newEndRow = startRow+5000            
            rows.MergeFrom(self.__getRows(plantId, wsType, startRow, newEndRow))
            startRow = newEndRow+1                
        rows.MergeFrom(self.__getRows(plantId, wsType, startRow, endRow))       
        return rows.items


import requests
import json
from datetime import datetime, time, timedelta
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_interfaces import user_pb2 as User
from one_interfaces import role_pb2 as role

import google
 
class AuthenticationApi:
	def __init__(self, env):
		self.Environment = env
		self.Token = Token()
		self.UserName = ""
		self.Password = ""
		self.User:User = User.User()
		self.IsAuthenticated =False	         

	def GetToken(self, user, password):
		data ={'username': user, 'password':password, 'grant_type':'password', 'scope':'FFAccessAPI openid', 'client_id':'VSTestClient', 'client_secret':'0CCBB786-9412-4088-BC16-78D3A10158B7'}
		headers = {'Accept': 'application/json', 'Content-Type': 'application/x-www-form-urlencoded'}
		url = self.Environment+"/connect/token"
		response= requests.post(url, headers=headers, data=data)
		if (response.status_code !=200):
			return ''
		self.__setToken(response)  		
		self.IsAuthenticated= True
		self.UserName =user
		self.Password = password		
		return self.Token.access_token

	def GetUserInfo(self):
		headers = {'Accept': 'application/json', "Authorization": self.Token.access_token}
		url = self.Environment+"/connect/userinfo"
		response= requests.get(url, headers=headers)		
		self.__setInfo(response)
		return response.content		

	def LoginResourceOwner(self, userName, password):
		data ={'username': userName, 'password':password, 'grant_type':'password', 'scope':'FFAccessAPI openid', 'client_id':'VSTestClient', 'client_secret':'0CCBB786-9412-4088-BC16-78D3A10158B7'}
		headers = {'Accept': 'application/json', 'Content-Type': 'application/x-www-form-urlencoded'}
		url = self.Environment+"/connect/token"
		response= requests.post(url, headers=headers, data=data)
		if (response.status_code !=200):
			return False
		self.__setToken(response)
		self.IsAuthenticated= True
		self.UserName =userName
		self.Password = password		
		return True

	def Logout(self):
		headers = {'Accept': 'application/json', "Authorization": self.Token.access_token}
		url = self.Environment+"/account/logout"
		response= requests.post(url, headers=headers)
		self.Token = Token()
		self.UserName = ""
		self.Password = ""
		self.User:User = User.User()
		self.IsAuthenticated =False

	def __setInfo(self, response):
		jResponse = json.loads(response.content)		
		self.User.firstName.value = jResponse.get('given_name')
		self.User.lastName.value = jResponse.get('family_name')
		self.User.userName = jResponse.get('user_name')
		self.User.email.value = jResponse.get('email')
		self.User.tenantId = jResponse.get('ActiveTenantId')
		self.User.id = jResponse.get('sub')
  
	def __setToken(self, tokenResponse):
		token = Token()
		token.created =datetime.now() 				
		responseJson=json.loads(tokenResponse.content)
		self.IsAuthenticated= True		
		token.token_type = responseJson['token_type']
		token.scope = responseJson['scope']
		token.access_token = token.token_type+" "+ responseJson['access_token']		
		token.expires_in =token.created +timedelta(seconds = responseJson['expires_in'])
		self.Token =token  	
  	

  
    
class Token:
    def __init__(self):
        self.access_token:str 
        self.expires_in:float
        self.token_type:str
        self.scope:str
        self.created:datetime
    def __repr__(self):
        return "Access token: %s, Created on: %s, Expires in: %s, Token Type: %s, Scope: %s  " %(self.access_token, self.created, self.expires_in, self.token_type, self.scope)


from datetime import datetime
import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class HistorianApi:
    def __init__(self, env, auth:AuthenticationApi):
        self.Environment =env
        self.Authentication = auth
        self.AppUrl = "/historian/data/v1/"
        
    def GetHistorianData(self, twinRefId, date:datetime):
        url=f'{self.Environment}{self.AppUrl}{twinRefId}/{date}'
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}
        print(url)
        response = DeserializeResponse(requests.get(url, headers=headers))
        return response.content.historianDatas.items
    
    def GetHistorianDataRange(self, twinRefId, startTime:datetime, endTime: datetime):        
        url=f'{self.Environment}{self.AppUrl}{twinRefId}?startTime={startTime}&endTime={endTime}'
        print(url)
        headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        return response.content.historianDatas.items
    
        